@@ -315,6 +315,13 @@ def _browser_context(
     is_flag=True,
     help="Skip images that already exist",
 )
+@click.option(
+    "outputs",
+    "-o",
+    "--output",
+    help="Just take shots matching these output files",
+    multiple=True,
+)
 @browser_option
 @user_agent_option
 @reduced_motion_option
@@ -325,6 +332,7 @@ def multi(
     timeout,
     fail_on_error,
     noclobber,
+    outputs,
     browser,
     user_agent,
     reduced_motion,
@@ -366,6 +374,8 @@ def multi(
                 and pathlib.Path(shot["output"]).exists()
             ):
                 continue
+            if outputs and shot.get("output") not in outputs:
+                continue
             try:
                 take_shot(context, shot)
             except TimeoutError as e:
@@ -441,11 +451,17 @@ def accessibility(url, auth, output, javascript, timeout):
     default="-",
     help="Save output JSON to this file",
 )
+@click.option(
+    "-r",
+    "--raw",
+    is_flag=True,
+    help="Output JSON strings as raw text",
+)
 @browser_option
 @user_agent_option
 @reduced_motion_option
 def javascript(
-    url, javascript, input, auth, output, browser, user_agent, reduced_motion
+    url, javascript, input, auth, output, raw, browser, user_agent, reduced_motion
 ):
     """
     Execute JavaScript against the page and return the result as JSON
@@ -487,6 +503,9 @@ def javascript(
         page.goto(url)
         result = _evaluate_js(page, javascript)
         browser_obj.close()
+    if raw:
+        output.write(str(result))
+        return
     output.write(json.dumps(result, indent=4, default=str))
     output.write("\n")
 
@@ -608,6 +627,82 @@ def pdf(
         browser_obj.close()
 
 
+@cli.command()
+@click.argument("url")
+@click.option(
+    "-a",
+    "--auth",
+    type=click.File("r"),
+    help="Path to JSON authentication context file",
+)
+@click.option(
+    "-o",
+    "--output",
+    type=click.Path(file_okay=True, writable=True, dir_okay=False, allow_dash=True),
+    default="-",
+)
+@click.option("-j", "--javascript", help="Execute this JS prior to saving the HTML")
+@click.option(
+    "-s",
+    "--selector",
+    help="Return outerHTML of first element matching this CSS selector",
+)
+@click.option(
+    "--wait", type=int, help="Wait this many milliseconds before taking the snapshot"
+)
+@browser_option
+@user_agent_option
+def html(
+    url,
+    auth,
+    output,
+    javascript,
+    selector,
+    wait,
+    browser,
+    user_agent,
+):
+    """
+    Output the final HTML of the specified page
+
+    Usage:
+
+        shot-scraper html https://datasette.io/
+
+    Use -o to specify a filename:
+
+        shot-scraper html https://datasette.io/ -o index.html
+    """
+    url = url_or_file_path(url, _check_and_absolutize)
+    if output is None:
+        output = filename_for_url(url, ext="html", file_exists=os.path.exists)
+    with sync_playwright() as p:
+        context, browser_obj = _browser_context(
+            p, auth, browser=browser, user_agent=user_agent
+        )
+        page = context.new_page()
+        page.goto(url)
+        if wait:
+            time.sleep(wait / 1000)
+        if javascript:
+            _evaluate_js(page, javascript)
+
+        if selector:
+            html = page.query_selector(selector).evaluate("el => el.outerHTML")
+        else:
+            html = page.content()
+
+        if output == "-":
+            sys.stdout.write(html)
+        else:
+            open(output, "w").write(html)
+            click.echo(
+                "HTML snapshot of '{}' written to '{}'".format(url, output), err=True
+            )
+
+        browser_obj.close()
+
+
 @cli.command()
 @click.option(
     "--browser",
